/* --------------------------------------------------------------------------
**	Cirrus Logic Laguna 546x chipset driver
**      Text 16 functions
** --------------------------------------------------------------------------
**
**	Copyright (C) 1997 Rodolphe Ortalo, [ortalo@laas.fr]
**
**	This program is free software; you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation; either version 2, or (at your option)
**	any later version.
**
**	You should have received a copy of the GNU General Public License
**	along with this program; see the file COPYING.  If not, write to
**	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
** ---------------------------------------------------------------------------
**	$Log: 546x_text16.inc,v $
**	Revision 1.5  1997/12/14 21:55:35  becka
**	uint/sint namespace fixed ... please test ...
**	
**	Revision 1.4  1997/11/27 22:22:45  ortalo
**	Changed all the old long macro definitions to shorter ones. (Register
**	names now fit my own coding style.)
**	Several clean-ups in chipset code, but no functional improvements.
**	Still some cleaning to do.
**	
**	Revision 1.3  1997/09/15 21:52:13  ortalo
**	Added set_font functions (adapted from the vga driver). Text mode restore
**	after a graphics mode set now works (thanks to all).
**	
**	Revision 1.2  1997/09/14 19:05:11  ortalo
**	Graphic mode setting and frame buffer MMIO access works (incorrectly
**	but... well...). The return to text mode after a graphic mode set
**	does not work ! (Either a bug, or the absence of set font functions).
**	
**	Revision 1.1  1997/08/31 18:57:14  ortalo
**	Text 9x16 mode setting works. Added a separate file for text functions.
**	Added set_origin/split functions.
**	
*/

/*
** Font setting
*/

typedef struct text16_font_reg_s 
{
	uchar SR2,SR4;
	uchar GR4,GR5,GR6;
} text16_font_reg;

static inline void text16_font_enter(text16_font_reg *regs) 
{
	IS_CRITICAL;

	ENTER_CRITICAL;

	regs->SR2 = CLIO_in8(SR2_PLANE_MASK);
	regs->SR4 = CLIO_in8(SR4_MEMORY_MODE);  /*  Save register state  */
	regs->GR4 = CLIO_in8(GR4_READ_MAP_SELECT);
	regs->GR5 = CLIO_in8(GR5_MODE);
	regs->GR6 = CLIO_in8(GR6_MISCELLANEOUS);

	CLIO_out8(SR2_PLANE_MASK, SR2_PLANE_2);	/* Font data goes to plane 2 */
	CLIO_out8(SR4_MEMORY_MODE, SR4_EXTENDED_MEMORY | SR4_ODD_EVEN);

	CLIO_out8(GR4_READ_MAP_SELECT, GR4_PLANE2); /* Read plane 2 */
	CLIO_out8(GR5_MODE, GR5_WRITE_MODE_0);
	CLIO_out8(GR6_MISCELLANEOUS, GR6_MEMORY_MAP_VGA | GR6_GRAPHICS_MODE);

	LEAVE_CRITICAL;
}

static void inline text16_font_leave(text16_font_reg *regs)
{
	IS_CRITICAL;

	ENTER_CRITICAL;

	CLIO_out8(SR2_PLANE_MASK, regs->SR2);
	CLIO_out8(SR4_MEMORY_MODE, regs->SR4);	/* Restore registers  */
	CLIO_out8(GR4_READ_MAP_SELECT, regs->GR4);
	CLIO_out8(GR5_MODE, regs->GR5);
	CLIO_out8(GR6_MISCELLANEOUS, regs->GR6);

	LEAVE_CRITICAL;
}

static inline void direct_text16_font_read(int chr, uchar *charcell)
{
/*
	memcpy(charcell, (uchar *)(phys_to_virt(0xA0000)) + (chr << 5), 32);
*/
	memcpy(charcell, (uchar *)(vga_mem_graph.base) + (chr << 5), 32);
}

static inline void direct_text16_font_write(int chr, uchar *charcell)
{
/*
	memcpy((uchar *)(phys_to_virt(0xA0000)) + (chr << 5), charcell, 32);
*/
	memcpy((uchar *)(vga_mem_graph.base) + (chr << 5), charcell, 32);
}

static void text16_do_set_font(int no, int from, int to, struct kgi_font *font)
{
	text16_font_reg fontregs;
	int i;
	char *fbuf, *data;
	register int cnt = 32 - font->dy;

	if (font == NULL) {
		NOTICE("NULL font pointer given to set_font");
		return;
	}

/*
	fbuf = (char *) phys_to_virt(0x000A0000);
*/
	fbuf = (char *) vga_mem_graph.base;
	fbuf += (no * 0x2000) + (from << 5);
	data = font->data + font->dy * from;
	if (!data)
		return;

	text16_font_enter(&fontregs);
	/* NOTE: no printk()'s are allowed in here unless klogd is active! */

	for (i = from; i <= to; i++) {
		memmove(fbuf, data, font->dy);
		memset(fbuf + font->dy, 0, cnt);
		fbuf += 32;
		data += font->dy;
	};

	text16_font_leave(&fontregs);

#ifdef 0 /* CURSOR_HW */
	/* Set up cursor */
	outbCRT((inbCRT(0x0A) & ~CR0A_CURSOR_START_MASK)
		| ((font->base+1) & CR0A_CURSOR_START_MASK), 0x0A);
	outbCRT((inbCRT(0x0B) & ~CR0B_CURSOR_END_MASK)
		| ((font->base+2) & CR0B_CURSOR_END_MASK), 0x0B);
#endif
}

static void text16_set_font(struct kgi_display *dpy, struct kgi_font *font,
			    ggi_uint from, ggi_uint to)
{
	text16_do_set_font(0, from, to, font);
}

static void text16_set_origin(struct kgi_display *dpy, ggi_uint x, ggi_uint y)
{
	struct chipset_private *private = KGIM_PRIVATE(dpy,chipset);
	IS_CRITICAL;

	register ggi_uint xpix = x / dpy->mode.request.text.x;
	register ggi_uint xdot = x % dpy->mode.request.text.x;
	register ggi_uint ypix = y / dpy->mode.request.text.y;
	register ggi_uint ydot = y % dpy->mode.request.text.y;
	register ggi_uint offset = xpix + ypix*dpy->fb.size.x;

	private->origin = offset;

	ENTER_CRITICAL;

		if (private->last_offset != offset) {

			CLIO_out8(CRC_SCREEN_START_ADDRESS_HIGH,
				(offset >> 8) & 0xFF);
			CLIO_out8(CRD_SCREEN_START_ADDRESS_LOW,
				 offset & 0xFF);
			private->last_offset = offset;
		}

		if (private->last_ydot != ydot) {

			CLIO_out8(CR8_SCREENA_PRESET_ROW_SCAN,
				 ydot & CR8_ROW_SCAN_MASK);

			/* NB: Byte pan is alway 0 (?) -- ortalo */
			private->last_ydot = ydot;
		}

		if (private->last_xdot != xdot) {

			register ggi_uint hpan;

			if (dpy->mode.request.text.x == 9) {

				hpan = (xdot == 0)
					    ? 0x08 : (xdot-1);
			} else {

				hpan = xdot;
			}
			/* This active wait should not be necessary
			** thanks to bit 1 ot CRT1A (see set_mode)
			*/
			#if 0
			wait_display_off(dpy);
			#endif

			CLIO_out8(AR13_PIXEL_PANNING,
				hpan & AR13_PIXEL_PANNING_MASK);

			private->last_xdot = xdot;
		}

	LEAVE_CRITICAL;
}

static void text16_set_splitline(struct kgi_display *dpy, int line)
{
	struct chipset_private *private = KGIM_PRIVATE(dpy,chipset);
	register uchar foo;

	if (private->splitline == line) {

		return;	
	}

	CLIO_out8(CR18_LINE_COMPARE, line & 0xFF);

	foo = (line & 0x0100)
		? CLIO_in8(CR7_OVERFLOW) | CR7_LINE_COMPARE_8
		: CLIO_in8(CR7_OVERFLOW) & ~CR7_LINE_COMPARE_8;
	CLIO_out8(CR7_OVERFLOW, foo);

	foo = (line & 0x0200)
		? CLIO_in8(CR9_CHARACTER_CELL_HEIGHT) | CR9_LINE_COMPARE_9
		: CLIO_in8(CR9_CHARACTER_CELL_HEIGHT) & ~CR9_LINE_COMPARE_9;
	CLIO_out8(CR9_CHARACTER_CELL_HEIGHT, foo);

	private->splitline=line;
}

static void text16ShowCursor(struct kgi_display *dpy, ggi_uint x, ggi_uint y)
{
	struct chipset_private *private = KGIM_PRIVATE(dpy,chipset);
	IS_CRITICAL;
	register ulong new = private->origin +
		(x / dpy->mode.request.text.x) +
		(y / dpy->mode.request.text.y) * dpy->fb.size.x;
/*
	#warning check fb_size for cursor -- ortalo
	new = (new < private->fb_size) ? new : new - private->fb_size;
	ASSERT(new < private->fb_size);
*/

	ENTER_CRITICAL;

		CLIO_out8(CRE_TEXT_CURSOR_LOCATION_HIGH, (new >> 8) & 0xFF);
		CLIO_out8(CRF_TEXT_CURSOR_LOCATION_LOW, new & 0xFF);

	LEAVE_CRITICAL;
}

static void text16HideCursor(struct kgi_display *dpy)
{
	IS_CRITICAL;
	ENTER_CRITICAL;

		CLIO_out8(CRE_TEXT_CURSOR_LOCATION_HIGH, 0xFF);
		CLIO_out8(CRF_TEXT_CURSOR_LOCATION_LOW, 0xFF);

	LEAVE_CRITICAL;
}

#define text16UndoCursor	text16HideCursor

static void text16ShowPointer(struct kgi_display *dpy, ggi_uint x, ggi_uint y)
{
	NOTICE2("Empty text16ShowPointer");
}

static void text16HidePointer(struct kgi_display *dpy)
{
	NOTICE2("Empty text16HidePointer");
}

#define text16UndoPointer	text16HidePointer
