/* --------------------------------------------------------------------------
**	TVP 3026 ramdac register definitions
** --------------------------------------------------------------------------
**
**	Copyright (C) 1997, Hartmut Niemann, Steffen Seeger
**
**	This program is free software; you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation; either version 2, or (at your option)
**	any later version.
**
**	You should have received a copy of the GNU General Public License
**	along with this program; see the file COPYING.  If not, write to
**	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
** ---------------------------------------------------------------------------
**
**	MAINTAINER	Hartmut_Niemann
**
**	$Log: tvp3026.h,v $
**	Revision 1.5  1997/12/03 15:48:03  seeger
**	added register names to be consisent with ATT DAC.
**	
**	Revision 1.4  1997/11/09 12:01:23  jonas_b
**	Just changed a variable name.
**	
**	Revision 1.3  1997/10/13 19:46:52  seeger
**	Removed DAC_REGISTERS definiton.
**	Added Bias defines for all PLL registers.
**	
**	Revision 1.2  1997/09/28 20:34:29  seeger
**	Added register bit definitions
**	
**	Revision 1.1  1997/08/19 21:27:50  seeger
**	Initial checkin of TVP 3026 register definitions by Hartmut Niemann.
**	adopted TLC 34076 header to 0.0.9 style.
**	
*/
#ifndef _TI_tvp3026_h
#define _TI_tvp3026_h

/*
**	Texas Instruments TVP3026 Register definitions
*/

#include "IBM/vga_dac.h"
/*	defines DAC_PW_INDEX,	RW palette/cursor ram write address,
**				index register
**		DAC_P_DATA,	RW palette RAM data
**		DAC_PIXEL_MASK,	RW pixel read mask
**		DAC_PR_INDEX	RW palette/cursor ram read address
*/

#define	DAC_EXT_ADDR		0x00

#define	DAC_CW_INDEX		0x04
#define	DAC_C_DATA		0x05
/* register reserved		0x06 */
#define	DAC_CR_INDEX		0x07

/* register reserved		0x08 */
#define	DAC_CURSOR_CTRL		0x09
	#define	DAC09_CursorOff		0x00
	#define	DAC09_CursorThreeColor	0x01
	#define	DAC09_CursorXGA		0x02
	#define	DAC09_CursorX		0x03
	#define	DAC09_CursorMask	0x03

#define	DAC_EXT_DATA		0x0A
#define	DAC_CURSOR_RAM_DATA	0x0B

#define	DAC_CURSOR_XL		0x0C
#define	DAC_CURSOR_XH		0x0D
#define	DAC_CURSOR_YL		0x0E
#define	DAC_CURSOR_YH		0x0F

/*
**	indexed registers
*/

/* register reserved			0x00 */
#define	DAC_EXT_Revision		0x01
	#define	DAC_EXT01_MinorMask	0x0F
	#define	DAC_EXT01_MinorShift	0
	#define	DAC_EXT01_MajorMask	0xF0
	#define	DAC_EXT01_MajorShift	4	
/*
**					0x02
** registers reserved			...
**					0x05
*/
#define	DAC_EXT_CursorControl		0x06
	#define	DAC_EXT06_CursorOff		0x00
	#define	DAC_EXT06_CursorThreeColor	0x01
	#define	DAC_EXT06_CursorXGA		0x02
	#define	DAC_EXT06_CursorX		0x03
	#define	DAC_EXT06_CursorMask	0x03
	
	#define	DAC_EXT06_CursorAddr8	0x04
	#define	DAC_EXT06_CursorAddr9	0x08
	#define	DAC_EXT06_4096Blank		0x10
	#define	DAC_EXT06_InterlacedCursor	0x20
	#define	DAC_EXT06_InterlaceInvert	0x40
	#define	DAC_EXT06_DirectCCR		0x80

/*
**					0x07
** registers reservd			...
**					0x0E
*/
#define	DAC_EXT_LatchControl		0x0F
/*
**					0x10
** registers reserved			...
**					0x17
*/
#define	DAC_EXT_TrueColorControl	0x18
#define	DAC_EXT_MutiplexControl		0x19
#define	DAC_EXT_ClockSelect		0x1A
	#define	DAC_EXT1A_VCLKMask	0x70
	#define	DAC_EXT1A_VCLKShift	4
	#define	DAC_EXT1A_ClkMask	0x0F
	#define	DAC_EXT1A_ClkShift	0
	#define	DAC_EXT1A_Clk0		0x00
	#define	DAC_EXT1A_Clk1		0x01
	#define	DAC_EXT1A_Clk2		0x02
	#define	DAC_EXT1A_Clk2Neg	0x03
	#define	DAC_EXT1A_Clk2Ecl	0x04
	#define	DAC_EXT1A_ClkPClkPLL	0x05
	#define	DAC_EXT1A_ClkOff	0x06
	#define	DAC_EXT1A_Clk0VGA	0x07	
		
/* register reserved			0x1B */
#define	DAC_EXT_PalettePage		0x1C
#define	DAC_EXT_GeneralConrol		0x1D
	#define	DAC_EXT1D_InvHSYNC	0x01
	#define	DAC_EXT1D_InvVSYNC	0x02
	#define	DAC_EXT1D_BigEndian	0x08
	#define	DAC_EXT1D_BlankPedestral 0x10
	#define	DAC_EXT1D_SyncEnable	0x20
	#define	DAC_EXT1D_OverscanEnable 0x40

#define	DAC_EXT_MiscControl		0x1E
	#define	DAC_EXT1E_PowerDown	0x01
	#define	DAC_EXT1E_6n8Disable	0x04
	#define	DAC_EXT1E_8BitEnable	0x08
	#define	DAC_EXT1E_PortSwitchEnable 0x10
	#define	DAC_EXT1E_PortSwitchInvert 0x20
/*
**					0x1F
** registers reserved			...
**					0x29
*/
#define	DAC_EXT_GIOControl		0x2A
#define	DAC_EXT_GIOData			0x2B
#define	DAC_EXT_PLLAddress		0x2C
	#define	DAC_EXT2C_PClkMask	0x03
	#define	DAC_EXT2C_PClkShift	0

	#define	DAC_EXT2C_MClkMask	0x0C
	#define	DAC_EXT2C_MClkShift	2

	#define	DAC_EXT2C_LClkMask	0x30
	#define	DAC_EXT2C_LClkShift	4

#define	DAC_EXT_PClkData		0x2D
#define	DAC_EXT_MClkData		0x2E
#define	DAC_EXT_LClkData		0x2F
/*
**	generic PLL definitions
*/
#define	DAC_PLL_N			0x00
#define	DAC_PLL_M			0x01
#define	DAC_PLL_P			0x02
#define	DAC_PLL_Status			0x03

#define	DAC_PClkN_Bias			0xC0

#define	DAC_PClkM_Bias			0x00

#define	DAC_PClkP_PLLEnable		0x80
#define	DAC_PClkP_PClkEnable		0x40
#define	DAC_PClkP_LForce		0x08
#define	DAC_PClkP_PForce		0x04
#define	DAC_PClkP_PMask			0x03
#define	DAC_PClkP_PShift		0
#define	DAC_PClkP_Bias			0x30

#define	DAC_PClkS_Lock			0x40


#define	DAC_MClkP_PLLEnable		0x80
#define	DAC_MClkP_LForce		0x08
#define	DAC_MClkP_PForce		0x04
#define	DAC_MClkP_PMask			0x03
#define	DAC_MClkP_PShift		0
#define	DAC_MClkP_Bias			0x30

#define	DAC_MClkS_Lock			0x40


#define	DAC_LClkN_Bias			0xC0

#define	DAC_LClkM_Bias			0x00

#define	DAC_LClkP_PLLEnable		0x80
#define	DAC_LClkP_LForce		0x08
#define	DAC_LClkP_PForce		0x04
#define	DAC_LClkP_PMask			0x03
#define	DAC_LClkP_PShift		0
#define	DAC_LClkP_Bias			0x70

#define	DAC_LClkS_Lock			0x40


#define	DAC_EXT_ColKeyOverlayL		0x30
#define	DAC_EXT_ColKeyOverlayH		0x31
#define	DAC_EXT_ColKeyRedL		0x32
#define	DAC_EXT_ColKeyRedH		0x33
#define	DAC_EXT_ColKeyGreenL		0x34
#define	DAC_EXT_ColKeyGreenH		0x35
#define	DAC_EXT_ColKeyBlueL		0x36
#define	DAC_EXT_ColKeyBlueH		0x37
#define	DAC_EXT_ColKeyControl		0x38
	#define	DAC_EXT38_CompareOverlay 0x01
	#define	DAC_EXT38_CompareRed	0x02
	#define	DAC_EXT38_CompareGreen	0x04
	#define	DAC_EXT38_CompareBlue	0x08
	#define	DAC_EXT38_InverseColKey	0x10
	#define	DAC_EXT38_ZoomMask	0xE0
	#define	DAC_EXT38_ZoomShift	5

#define	DAC_EXT_ClkControl		0x39
	#define	DAC_EXT39_MClkDivMask	0x07
	#define	DAC_EXT39_MClkDivShift	0
	#define	DAC_EXT39_MClkOutStrobe	0x08
	
	#define	DAC_EXT39_MClkOutDClk	0x00
	#define	DAC_EXT39_MClkOutMClk	0x10
	#define	DAC_EXT39_MClkOutMask	0x10

	#define	DAC_EXT39_PClkOutPClk	0x00
	#define	DAC_EXT39_PClkOutLClk	0x20
	#define	DAC_EXT39_PClkOutDClk	0x40	
	#define	DAC_EXT39_PClkOutMask	0x60

#define	DAC_EXT_SenseTest		0x3A
#define	DAC_EXT_TestModeData		0x3B
#define	DAC_EXT_CRCRemainder		0x3C

#define DAC_REV                         0x01
#define DAC_EXT3F_ID                    0x3F
#define DAC_EXT3F_TVP3026_ID            0x26
#endif	/* #ifdef _TI_tvp3026.h */
