/* --------------------------------------------------------------------------
**	Tseng ET4000 chipset driver
** --------------------------------------------------------------------------
**
**	Copyright (C) 1997      Andreas Vogler
**
**	This program is free software; you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation; either version 2, or (at your option)
**	any later version.
** 
**	You should have received a copy of the GNU General Public License
**	along with this program; see the file COPYING.  If not, write to
**	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
**
** ---------------------------------------------------------------------------
**	This code is derived from the following source of information:
**
**	[1]	IBM (vga) chipset driver by Jason McMullan
**	[2]	VGADOC 4/WHATVGA 2.00 by Finn Thoegersen
**
*/

#ifndef _Tseng_et4000_h
#define	_Tseng_et4000_h

#include "IBM/vga.h"

/* additional registers for the et4000 family */

/* ATC */ 
	#define ATC_MISC	0x16
		#define AR16_DISBL_OVERSCANREG	0x01
		#define AR16_DISBL_PLRAM	0x02
		#define AR16_HIRES_MASK		0x30 /* diff.meaning for W32!*/
		#define AR16_W32_HICOLOR	0x20
		#define AR16_HIRES_8BIT		0x20
		#define AR16_HIRES_16BIT	0x30
		#define AR16_ENBL_2BYTE		0x40
		#define AR16_DISBL_EGAPAL	0x80
	#define ATC_MISC1	0x17
		#define AR17_REDEFINE_ATR	0x80

/* SEQ */
	#define SEQ_TSCTRL	0x06
		#define SR06_WIDTH_MASK	0x06
	#define SEQ_TSAUX	0x07
		#define SR07_DIV4	0x01
		#define SR07_EQ_CLKS	0x02
		#define SR07_SCALE_MASK	0x14	/* ET4000W32x ONLY !!! */
		#define SR07_ROM_MASK	0x28
		#define SR07_DIV2	0x40
		#define SR07_ENBL_VGA	0x80

/* Microchannel Video Subsystem Enable Register */
#define MVSE_REG	0x3C3

/* Extended Bank Registers */
#define EXT_BANK	0x3CB		/* W32 ONLY !! */

/* Segment Select */
#define SEG_SELECT	0x3CD
		#define SGS_WRITE_MASK	0x0F
		#define SGS_READ_MASK	0xF0

/* CRT color */
	#define CRT_LFB		0x30	/* Linear frame buffer; W32 ??? */
	#define CRT_GENERAL	0x31
		#define CR31_CLK_MASK	0xC0
	#define CRT_RASCAS	0x32 
		#define CR32_INTERLEAVE	0x80
	#define CRT_EXT_ADDR	0x33
		#define CR33_DISPLAYSTART_MASK	0x03
		#define CR33_DS_MASK_W32X	0x0F
		#define CR33_CURSORSTART_MASK	0x0C
		#define CR33_CS_MASK_W32X	0xF0
	#define CRT_CCR		0x34
		#define CR34_CLK_SEL2	0x02
	#define CRT_OVRFLW_V	0x35
		#define CR35_VBLANKSTART_B10	0x01
		#define CR35_VTOTAL_B10	0x02
		#define CR35_VDISPLAYEND_B10	0x04
		#define CR35_VSYNCSTART_B10	0x08
		#define CR35_LINECOMPARE_B10	0x10
		#define CR35_INTERLACE		0x80
	#define CRT_VIDEO_CFG1	0x36
	#define CRT_VIDEO_CFG2	0x37
	#define CRT_OVRFLW_H	0x3F
		#define CR3F_HTOTAL_B8	0x01
		#define CR3F_HBLANKSTART_B8	0x04
		#define CR3F_HSYNCSTART_B8	0x10
		#define CR3F_LOGICALWIDTH_B8	0x80

/* AT&T mode control */
#define ATT_CTRL	0x3DE

/* Display mode control */
#define DSPLY_MODE	0x3D8

/* CRTCB/Sprite register at 0x21xA; it is usually at 0x217A */
#define SPRITEI		0x217A
#define SPRITED		0x217B

#endif	/* #ifndef _Tseng_et4000_h */

