/* 
   Graphics library for GGI. 
   Generic Gamma correction library

   Copyright (C) 1997 Jason McMullan - jmcc@cs.cmu.edu

   If you do any modifications, I would like you to send diffs to me
   to allow for collecting a more and more complete set of drivers and
   to improve existing ones.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <math.h>
#include <sys/ioctl.h>

#ifdef _AIX
#include <sys/types.h>
#include <unistd.h>
#endif

#include "ggi-dl.h"
#include <ggi/ggi_commands.h>

extern ggi_color *_GGIcolormap;
extern int _GGInocols;

static struct {
	int set;
	ggi_float r,g,b;
} _gamma = {0,};

int GGIsetgammamap(ggi_visual *vis,int start,int len,ggi_color *colormap)
{
	if (colormap==NULL || start+len >_GGInocols)
		return -1;

	if (_GGIcolormap+start != colormap) {
		/* Optimize for our internal use.. */
		memcpy(_GGIcolormap+start,colormap,len*sizeof(ggi_color));
	}

	return ioctl(vis->fd,(int)RAMDAC_SETCLUT,_GGIcolormap);
}

int GGIgetgammamap(ggi_visual *vis,int start,int len,ggi_color *colormap)
{
	if (colormap==NULL || start+len >_GGInocols)
		return -1;

	/* Optimize for our internal use.. */
	if (_GGIcolormap+start == colormap)
		return 0;

	memcpy(colormap,_GGIcolormap+start,len*sizeof(ggi_color));

	return 0;
}


int GGIgetgamma(ggi_visual *vis,ggi_float *r,ggi_float *g,ggi_float *b)
{
	if (!_gamma.set) 
		return -1;

	(*r)=_gamma.r;
	(*g)=_gamma.g;
	(*b)=_gamma.b;

	return 0;
}

int GGIsetgamma(ggi_visual *vis,ggi_float r,ggi_float g,ggi_float b)
{
	ggi_float inten,delta;
	int i;

	if (r<=0.0 || g<=0.0 || b<=0.0)
		return -1;

	/* Set up the gamma structs.. */
	_gamma.set=1;
	_gamma.r=r;
	_gamma.g=g;
	_gamma.b=b;

	/* inverse the gamma values */
	r=1.0/r;
	g=1.0/g;
	b=1.0/b;

	delta=1.0/_GGInocols;
	inten=0.0;
	for (i=0;i<_GGInocols;i++) {
		inten += delta;

		_GGIcolormap[i].r=pow(inten,r);
		_GGIcolormap[i].g=pow(inten,g);
		_GGIcolormap[i].b=pow(inten,b);

	}
	
	ggiSetGammaMap(vis,0,_GGInocols,_GGIcolormap);

	return 0;
} 
