/* 
   Graphics library for GGI. Pixels.

   Copyright (C) 1997 Jason McMullan - jmcc@cs.cmu.edu

   If you do any modifications, I would like you to send diffs to me
   to allow for collecting a more and more complete set of drivers and
   to improve existing ones.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "ggi-dl.h"

/* Return `BW' mapping
 */
ggi_uint GGImapcolor(ggi_visual *vis,ggi_color col)
{
	int intensity;

	/* If more that two colors are brighter than
	 * mid-intenisty, turn on the intensity bit..
 	 */
	intensity=(col.r+col.b+col.g);
	intensity = (intensity >= 2*0x8000) ? 1 : 0;

	return	intensity;
}

/* Map `BW' pixel to color
 */
int GGIunmappixel(ggi_visual *vis,ggi_uint pixel,ggi_color *col)
{
	if (pixel > 1)
		return -1;

	if (pixel&1) 
		col->r=col->g=col->b=0xFFFF;
	else
		col->r=col->g=col->b=0;

	return EOK;	
}

/* Pack the colors into an array
 */
int GGIpackcolors(ggi_visual *vis,void *outbuf,ggi_color *cols,int len)
{
	uchar tmp=0,*obuf=(uchar *)outbuf;
	int mask,i;

	mask=7;
	for (i=0;i<len;i++) {
		tmp |= ggiMapColor(vis,*(cols++)) << mask--;
		if (mask<0) {
			*(obuf++)=tmp;
			tmp=0;
			mask=7;
		}
	}

	return 0;
}	
	
/* Unpack into the ggi_color array the values of the pixels
 */
int GGIunpackpixels(ggi_visual *vis,void *inbuf,ggi_color *cols,int len)
{
	uchar *ibuf=(uchar *)inbuf;
	int i,mask;
	ggi_uint tmp;

	mask=7;
	for (i=0;i<len;i++) {
		tmp=(*ibuf) >> mask--;
		ggiUnmapPixel(vis,tmp,(cols++));
		if (mask<0) {
			ibuf++;
			mask=7;
		}
	}
	return 0;
}	
	
  
