#ifndef _LIBGGI_DIRECTBUFFER_H_
#define _LIBGGI_DIRECTBUFFER_H_

/*
 * (c) 1997 Andreas Beck becka@sunserver1.rz.uni-duesseldorf.de
 * (c) 1997 Jason McMullan jmcc@ontv.com
 * (c) 1997 Steffen Seeger seeger@physik.tu-chemnitz.de
 * ------------------------------------------------------------------------
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU Library General Public License as 
 *	published by the Free Software Foundation; either version 2, or 
 *	(at your option) any later version.
 * 
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Library General Public License for more details.
 *
 *	You should have received a copy of the GNU Library General Public 
 *	License	along with this program; see the file COPYING.LGPL .  
 *	If not, write to the Free Software Foundation, 675 Mass Ave, 
 *	Cambridge, MA 02139, USA.
 *
 * ------------------------------------------------------------------------
 */

/*	Pixel Attributes	*/
typedef enum ggi_pixel_attribute {

	paPrivate,		/* Don't touch				P */
	paApplication,		/* store what you want in here		A */
	paBackgroundIndex,	/* index of the color of the pixel	i */
	paForegroundIndex,	/* index of the color of the texture	T */
	paTextureIndex,		/* index of the texture			t */
	paBlink,		/* blink bit/frequency			B */
	paColor1,		/* intensity of color channel 1		r */
	paColor2,		/* intensity of color channel 2		g */
	paColor3,		/* intensity of color channel 3		b */
	paAlpha,		/* alpha value (all channels)		a */
	paAlpha1,		/* alpha value channel 1		1 */
	paAlpha2,		/* alpha value channel 2		2 */
	paAlpha3,		/* alpha value channel 3		3 */
	paZValue,		/* z-value				z */
	paStencil,		/* stencil buffer value			s */
	paBlendKey,		/* a key used for blending		*/
	paBlendLow1,		/* keying low value channel 1		*/
	paBlendHigh1,		/* keying high value channel 1		*/
	paBlendLow2,		/* keying low value channel 2		*/
	paBlendHigh2,		/* keying high value channel 2		*/
	paBlendLow3,		/* keying low value channel 3		*/
	paBlendHigh3,		/* keying high value channel 3		*/

	paLastPixelAttribute	/* the last defined, must(!) be less 32	*/

} ggi_pixel_attribute;

/* Common setups for pixel-linear-buffers. 
 * Values here are sp[bit-stride]a[acces/alignment widths][bitmasks]
 * Where bitmasks is a string lowbit->highbit,lowbyte->highbyte with the 
 * above letters.
 */
typedef enum ggi_common_plb_setup {

	spUncommon,		/* The layout is not near any defined standard */

	sp1a8lbl		=0x01000000,		/* Mono, lowbit left */
	sp1a8hbl,					/* Mono, highbit left */

	sp2dummy		=0x02000000,		/* No known 2 bit modes yet */

	sp4a8lnl		=0x04000000,		/* 4 bit, low nibble left */
	sp4a8hnl,					/* 4 bit, high nibble left */

	sp8a8i8			=0x08000000,		/* 8 bit indexed mode (VGA palette) */

	sp16a16r5g5b5A1		=0x10000000,		/* 15 bit RGB 5/5/5 mapping lit endian */
	sp16a16r5g5b5A1be,				/* 15 bit RGB 5/5/5 mapping big endian */

	sp16a16r5g6b5		=0x11000000,		/* 16 bit RGB 5/6/5 mapping lit endian */
	sp16a16r5g6b5be,				/* 16 bit RGB 5/6/5 mapping big endian */

	sp24a8b8g8r8		=0x18000000,		/* 24 bit BGR */
	sp24a8r8g8b8,					/* 24 bit RGB */

	sp32a32b8g8r8A8		=0x20000000,		/* RGBx as used in S3 on lit-endian machines */
	sp32a32A8r8g8b8,				/* RGBx as used in S3 on big-endian machines */

	spLastSetup		/* the last defined */
} ggi_common_plb_setup;

enum ggi_buffer_layout {

	blPixelLinearBuffer,

	blLastBufferLayout
};

typedef struct ggi_pixelattributedesc {

	ggi_pixel_attribute
		 left_frame_last_attr;
	ggi_uint	*left_frame_attr_mask;

	ggi_pixel_attribute
		 right_frame_last_attr;
	ggi_uint	*right_frame_attr_mask;

} ggi_pixelattributedesc;

typedef struct ggi_pixellinearbuffer
{
	void	*read;		/* buffer address for reads	*/
	void	*write;		/* buffer address for writes	*/
	ggi_uint	page_size;	/* zero for true linear buffers	*/

	ggi_uint	access;		/* supported access widths	*/
	ggi_uint	align;		/* alignment requirements	*/
	ggi_uint	swap;		/* swapping requirements	*/

	uintl	write_mask;

	/*	buffer layout	*/
	ggi_uint	bpp;		/* bits per pixel		*/
	ggi_sint	stride;		/* pixels per row		*/
	ggi_coord	origin; /* application area origin	*/
	ggi_coord	size;	/* size of application area	*/

	char 	colorstring[17];	/* Define the meaning of the paColor? attributes. */

	/* Enumerator for often used modes for fast checks. */
	ggi_common_plb_setup	setup;	

	ggi_uint	numpad;		/* number of PADs */

	ggi_pixelattributedesc	pad[1];	/* allocate struct longer if necessary */

} ggi_pixellinearbuffer;

/* Annotation : known colorstrings are :
 *              - RGB
 *              - CMY
 *              - CMYK
 *              - K (greyscale)
 *		- I (indexed)
 *              - YUV
 *              - YCC
 */


typedef struct ggi_directbuffer
{
	/*	administrative. DONT TOUCH !	*/
	struct ggi_directbuffer	*next;

	void	*app_private;	/* Steffen : what are these for ? */
	void	*lib_private;
	void	*drv_private;

	/*	access info	*/
	enum ggi_buffer_layout	layout;

	/* The actual buffer info. Depends on layout. */
	union {
		ggi_pixellinearbuffer pixellinearbuffer;
	}			buffer;

} ggi_directbuffer;


#ifndef _BUILDING_LIBGGI
typedef void *ggi_directbuffer_t;
#else
typedef ggi_directbuffer *ggi_directbuffer_t;
#endif

/* DB management */
int  ggiDBGetBuffer(ggi_visual_t vis, ggi_directbuffer_t *buf);

enum ggi_buffer_layout ggiDBGetLayout(ggi_directbuffer_t buf);

ggi_pixellinearbuffer *ggiDBGetPLB   (ggi_directbuffer_t buf);

ggi_uint ggiDB_PLBAttributeMask(ggi_visual_t vis, ggi_pixellinearbuffer *buf,
			ggi_uint eye, ggi_uint frame,
			enum ggi_pixel_attribute attr);

#endif /* _LIBGGI_DIRECTBUFFER_H_ */
