/* 
   Graphics library for GGI. Horizontal lines.

   Copyright (C) 1995 Andreas Beck - becka@hp.rz.uni-duesseldorf.de

   If you do any modifications, I would like you to send diffs to me
   to allow for collecting a more and more complete set of drivers and
   to improve existing ones.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <string.h>

#include "ggi-dl.h"
#include "vga4.h"

/**********************************/
/* draw/get/put a horizontal line */
/**********************************/

/* Worst case - 1 page fault
 */
int GGIdrawhline(ggi_visual *vis,int x,int y,int w)
{
	volatile unsigned char *adr;int i,j;
	unsigned char foo;

	CHECKXYW(vis,x,y,w);

	adr=__localfb;
	adr+=((VIS_GC_FGCOLOR(vis)&0xF) << 16);
	adr+=(x+y*VIS_FB_WIDTH(vis))>>3;

	j=w;
	if ((i=x&7)) {
		if ((j=j+i-8)<=0){
			foo=*adr;
			*(adr++)=((0xff>>i)&(0xff<<-j));
			return EOK;
		} else {
			foo=*adr;
			*(adr++)=(0xff>>i);
		}
	} 

	while ((j-=8)>=0)
		*(adr++)=0xff;

	foo=*adr;
	*adr=~(0xff>>(j&7));

	return EOK;
}

/*
 * Worst case: 16(!) page faults
 */
int GGIputhline(ggi_visual *vis,int x,int y,int w,void *buffer)
{ 
	unsigned char *adr,*ptr,*buff=(unsigned char *)buffer;
	int mask,sw,i;
	volatile register unsigned char foo;

	CHECKXYW(vis,x,y,w);

	sw=(VIS_FB_WIDTH(vis)>>3);
	adr=__localfb+((x>>3)+y*sw);

	for (mask=0;mask<16;mask++) {
		ptr=adr+(mask<<16);
		buff=(unsigned char *)buffer;
		for (i=0;i<w;i++) {
		  	if ( ((buff[i>>1] >> ( (i&1) ? 4 : 0)) & 0xf) == mask) {
				foo=*ptr;
				*ptr=(0x80>>((x+i)&7));
			}
			if (((x+i)&7)==7) ptr++;
		}
	}
		
	return EOK;
}

/*
 * Worst case: 4(!) page faults
 */
int GGIgethline(ggi_visual *vis,int x,int y,int w,void *buffer)
{ 
	unsigned char *adr,*ptr,*buff=(unsigned char *)buffer;
	unsigned int plane,i,temp;

	CHECKXYW(vis,x,y,w);

	adr=__localfb+y*(VIS_FB_WIDTH(vis)>>3)+(x>>3);

	memset(buffer,0,(w+1)<<2);

	for (plane=0;plane<4;plane++) {
		ptr=adr+(plane<<16);
		buff=(unsigned char *)buffer;
		temp=*(ptr++)<<(x&7);
		for (i=0;i<w;i++) {
			buff[i>>1]|=((temp&0x80)>>(7-((i&1)<<2)-plane));
			temp<<=1;
			if (((x+i)&7)==7) temp=*(ptr++);
		}
	}
		
	return EOK;
}
