/* 
   Graphics library for GGI.

   Copyright (C) 1995 Andreas Beck - becka@hp.rz.uni-duesseldorf.de
   Copyright (C) 1997 Jason McMullan - jmcc@cs.cmu.edu

   If you do any modifications, I would like you to send diffs to me
   to allow for collecting a more and more complete set of drivers and
   to improve existing ones.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>

#include "ggi-dl.h"
#include <ggi/ggi_commands.h>
#include <IBM/modes.h>

/* 
 * We are guaranteed to have been checked already...
 */

unsigned char *__localfb=NULL;
static int __localfb_len;
static ggi_gc *_ggiGC=NULL;

int GGIdlinit(ggi_visual *visual,const char *version)
{
	__localfb_len=16*0x10000;

	__localfb=mmap(NULL,
			__localfb_len, /* 16*64k */
			PROT_READ|PROT_WRITE,
			MAP_SHARED,
			VIS_FD(visual),
			MMAP_TYPE_MMIO|
			MMAP_PER_REGION_TYPE|MMAP_IBM_VGA_4);
	if (__localfb==NULL ||
	    (long)(__localfb) == -1) 
		return -1;

	if (VIS_GC(visual)==NULL) {
		_ggiGC=malloc(sizeof(ggi_gc));
		if (_ggiGC==NULL) {
			if (__localfb!=NULL)
				free(__localfb);
			return -1;
		}
		_ggiGC->version=0;
/*		_ggiLockInit(_ggiGC);*/
	}

	if (VIS_GC(visual)==NULL)
		VIS_GC(visual)=_ggiGC;

	/* Color mapping 
	 */

	/* Positioning is not supported...
	 */

	/* Sprites aren't supported...
	 */

	/* Generic drawing
	 */
	visual->op2d->fillscreen=GGIfillscreen;

	visual->op2d->putc=GGIputc;
	visual->op2d->puts=GGIputs;

	visual->op2d->drawpixel=GGIdrawpixel;
	visual->op2d->putpixel=GGIputpixel;
	visual->op2d->getpixel=GGIgetpixel;

	visual->op2d->drawhline=GGIdrawhline;
	visual->op2d->puthline=GGIputhline;
	visual->op2d->gethline=GGIgethline;

	visual->op2d->drawvline=GGIdrawvline;
	visual->op2d->putvline=GGIputvline;
	visual->op2d->getvline=GGIgetvline;

	visual->op2d->drawbox=GGIdrawbox;
	visual->op2d->putbox=GGIputbox;
	visual->op2d->getbox=GGIgetbox;

	return GGI_DL_OPCOLOR|GGI_DL_OP2D;
}

int GGIdlcleanup(ggi_visual *visual)
{
	if (__localfb!=NULL) {
		munmap(__localfb,__localfb_len);
	}

	if (VIS_GC(visual)==_ggiGC) {
/*		_ggiLockDestroy(VIS_GC(visual));*/
		free(VIS_GC(visual));
		VIS_GC(visual)=NULL;
		_ggiGC=NULL;
	}

	return EOK;
}
		

