/* 
   Graphics library for GGI. Vertical lines.

   Copyright (C) 1995 Andreas Beck - becka@hp.rz.uni-duesseldorf.de

   If you do any modifications, I would like you to send diffs to me
   to allow for collecting a more and more complete set of drivers and
   to improve existing ones.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <string.h>

#include "ggi-dl.h"
#include "vga4.h"

/**********************************/
/* draw/get/put a vertical line */
/**********************************/

/* Worst case - 1 page fault
 */
int GGIdrawvline(ggi_visual *vis,int x,int y,int h)
{
	volatile unsigned char *adr;
	int i,sw,bm;
	unsigned char foo;

	CHECKXYH(vis,x,y,h);

	bm=(0x80>>(x&7));
	sw=(VIS_FB_WIDTH(vis))>>3;
	adr=(__localfb);
	adr+=((VIS_GC_FGCOLOR(vis)&0xF) << 16);
	adr+=(x>>3)+y*sw;

	for (i=h;i--;adr+=sw) {
		foo=*adr;
		*adr=bm;
	};
  
	return EOK;
}

/*
 * Worst case: 16(!) page faults
 */
int GGIputvline(ggi_visual *vis,int x,int y,int h,void *buffer)
{ 
	unsigned char *adr,*ptr,*buff=(unsigned char *)buffer;
	int mask,sw,i,bm;
	volatile register unsigned char foo;

	CHECKXYH(vis,x,y,h);

	sw=(VIS_FB_WIDTH(vis)>>3);
	adr=(__localfb);
	adr+=(x>>3)+y*sw;

	bm=(0x80>>(x&7));
	for (mask=0;mask<16;mask++) {
		ptr=adr+(mask<<16);
		buff=(unsigned char *)buffer;
		for (i=0;i<h;i++,ptr+=sw) {
			if ( (buff[i>>1] >> (i&1 ? 0 : 4)) == mask) {
				foo=*ptr;
				*ptr=bm;
			}
		}
	}
		
	return EOK;
}

/*
 * Worst case: 4 page faults
 */
int GGIgetvline(ggi_visual *vis,int x,int y,int h,void *buffer)
{ 
	unsigned char *adr,*ptr,*buff=(unsigned char *)buffer;
	int plane,sw,i,bm;

	CHECKXYH(vis,x,y,h);

	sw=(VIS_FB_WIDTH(vis)>>3);
	adr=__localfb;
	adr+=(x>>3)+y*sw;

	memset(buffer,0,(h+1)>>1);

	bm=(0x80>>(x&7));
	for (plane=0;plane<4;plane++) {
		ptr=adr+(plane<<16);
		buff=(unsigned char *)buffer;
		for (i=0;i<h;i++,ptr+=sw)
		  buff[i>>1]|=((*ptr&bm)!=0)<<((((i&1)^1)<<2)+plane);
	}
		
	return EOK;
}
